# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops, interface, port

import json
import logging


def update_port_loop_protect(interface_name, action=None, vlan_list=[], **kwargs):
    """
    Perform GET and PUT calls to apply Loop-protect options on an interface.

    :param interface_name: Alphanumeric String that is the name of the interface that will apply loop-protect options
    :param action: Alphanumeric String that will specify the actions for the Loop-protect interface.  The options are
        "do-not-disable", "tx-disable", "tx-rx-disable", or None.
    :param vlan_list: List of VLANs that will be configured for Loop-protect on the interface
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if action not in ['do-not-disable', 'tx-disable', 'tx-rx-disable', None]:
        raise Exception("ERROR: Action should be 'do-not-disable', 'tx-disable', 'tx-rx-disable' or None")

    if kwargs["url"].endswith("/v1/"):
        return _update_port_loop_protect_v1(interface_name, action, vlan_list, **kwargs)
    else:   # Updated else for when version is v10.04
        return _update_port_loop_protect(interface_name, action, vlan_list, **kwargs)


def _update_port_loop_protect_v1(interface_name, action="", vlan_list=[], **kwargs):
    """
    Perform GET and PUT calls to apply Loop-protect options on an interface.  Note that Loop-protect requires that
    the interface is L2, so this function will also update the interface to reflect that.

    :param interface_name: Alphanumeric String that is the name of the interface that will apply loop-protect options
    :param action: Alphanumeric String that will specify the actions for the Loop-protect interface.  The options are
        "do-not-disable", "tx-disable", "tx-rx-disable", or None.
    :param vlan_list: List of VLANs that will be configured for Loop-protect on the interface
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    port_name_percents = common_ops._replace_special_characters(interface_name)

    port_data = port.get_port(port_name_percents, depth=0, selector="configuration", **kwargs)

    # must remove these fields from the data since they can't be modified
    port_data.pop('name', None)
    port_data.pop('origin', None)

    port_data['loop_protect_enable'] = True
    # make interface L2
    port_data['routing'] = False

    # strings appended to output prints for status
    action_output = ""
    vlan_output = ""

    if action not in ['do-not-disable', 'tx-disable', 'tx-rx-disable', None]:
        raise Exception("ERROR: Action should be 'do-not-disable', 'tx-disable', 'tx-rx-disable' or None")
    elif action:
        port_data['loop_protect_action'] = action
        action_output = " with Action %s " % action

    if vlan_list:
        vlan_output = " with VLAN(s) ["
        for vlan in vlan_list:
            vlan_url = "/rest/v1/system/vlans/%s" % vlan
            if vlan_url not in port_data['loop_protect_vlan']:
                port_data['loop_protect_vlan'].append(vlan_url)
                vlan_output += (" " + str(vlan))
        vlan_output += "] "

    target_url = kwargs["url"] + "system/ports/%s" % port_name_percents
    put_data = json.dumps(port_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Applying Loop-protect to Interface '%s'%s%s failed with status code %d: %s"
              % (interface_name, action_output, vlan_output, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Applying Loop-protect to Interface '%s'%s%s succeeded"
              % (interface_name, action_output, vlan_output))
        return True


def _update_port_loop_protect(interface_name, action="", vlan_list=[], **kwargs):
    """
    Perform GET and PUT calls to apply Loop-protect options on an interface.  Note that Loop-protect requires that
    the interface is L2, so this function will also update the interface to reflect that.

    :param interface_name: Alphanumeric String that is the name of the interface that will apply loop-protect options
    :param action: Alphanumeric String that will specify the actions for the Loop-protect interface.  The options are
        "do-not-disable", "tx-disable", "tx-rx-disable", or None.
    :param vlan_list: List of VLANs that will be configured for Loop-protect on the interface
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    int_name_percents = common_ops._replace_special_characters(interface_name)
    int_data = interface.get_interface(int_name_percents, depth=1, selector="writable", **kwargs)

    if interface_name.startswith('lag'):
        if int_data['interfaces']:
            int_data['interfaces'] = common_ops._dictionary_to_list_values(int_data['interfaces'])

    if int_data['vlan_trunks']:
        int_data['vlan_trunks'] = common_ops._dictionary_to_list_values(int_data['vlan_trunks'])
    if int_data['loop_protect_vlan']:
        int_data['loop_protect_vlan'] = common_ops._dictionary_to_list_values(int_data['loop_protect_vlan'])

    int_data['loop_protect_enable'] = True
    # make interface L2
    int_data['routing'] = False

    # strings appended to output prints for status
    action_output = ""
    vlan_output = ""

    if action not in ['do-not-disable', 'tx-disable', 'tx-rx-disable', None]:
        raise Exception("ERROR: Action should be 'do-not-disable', 'tx-disable', 'tx-rx-disable' or None")
    elif action:
        int_data['loop_protect_action'] = action
        action_output = " with Action %s " % action

    if vlan_list:
        vlan_output = " with VLAN(s) ["
        for vlan in vlan_list:
            vlan_url = "/rest/v10.04/system/vlans/%s" % vlan
            if vlan_url not in int_data['loop_protect_vlan']:
                int_data['loop_protect_vlan'].append(vlan_url)
                vlan_output += (str(vlan) + " ")
        vlan_output += "] "

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Applying Loop-protect to Interface '%s'%s%s failed with status code %d: %s"
              % (interface_name, action_output, vlan_output, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Applying Loop-protect to Interface '%s'%s%s succeeded"
              % (interface_name, action_output, vlan_output))
        return True


def clear_port_loop_protect(port_name, **kwargs):
    """
    Perform GET and PUT calls to clear a Port's Loop-protect settings

    :param port_name: Alphanumeric name of the Port
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _clear_port_loop_protect_v1(port_name, **kwargs)
    else:   # Updated else for when version is v10.04
        return _clear_port_loop_protect(port_name, **kwargs)


def _clear_port_loop_protect_v1(port_name, **kwargs):
    """
    Perform GET and PUT calls to clear a Port's Loop-protect settings

    :param port_name: Alphanumeric name of the Port
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    port_name_percents = common_ops._replace_special_characters(port_name)

    port_data = port.get_port(port_name, depth=0, selector="configuration", **kwargs)

    port_data.pop('loop_protect_enable', None)
    port_data.pop('loop_protect_action', None)
    port_data['loop_protect_vlan'] = []

    # must remove these fields from the data since they can't be modified
    port_data.pop('name', None)
    port_data.pop('origin', None)

    target_url = kwargs["url"] + "system/ports/%s" % port_name_percents
    put_data = json.dumps(port_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Clearing Loop-protect options on Port '%s' failed with status code %d: %s"
              % (port_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Clearing the Loop-protect options on Port '%s' succeeded"
              % (port_name))
        return True


def _clear_port_loop_protect(interface_name, **kwargs):
    """
    Perform GET and PUT calls to clear an Interface's Loop-protect settings

    :param interface_name: Alphanumeric name of the Interface
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    int_name_percents = common_ops._replace_special_characters(interface_name)

    int_data = interface.get_interface(interface_name, depth=1, selector="writable", **kwargs)

    if interface_name.startswith('lag'):
        if int_data['interfaces']:
            int_data['interfaces'] = common_ops._dictionary_to_list_values(int_data['interfaces'])

    if int_data['vlan_trunks']:
        int_data['vlan_trunks'] = common_ops._dictionary_to_list_values(int_data['vlan_trunks'])

    int_data['loop_protect_enable'] = None
    int_data['loop_protect_action'] = "tx-disable"
    int_data['loop_protect_vlan'] = []

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Clearing Loop-protect options on Interface '%s' failed with status code %d: %s"
              % (interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Clearing the Loop-protect options on Interface '%s' succeeded"
              % (interface_name))
        return True



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops, port

import json
import random
import logging


def get_interface(int_name, depth=0, selector=None, **kwargs):
    """
    Perform a GET call to retrieve data for an Interface table entry

    :param int_name: Alphanumeric name of the interface
    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', or 'statistics'.  If running v10.04 or later, an additional option 'writable' is included.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing data for Interface entry
    """
    if kwargs["url"].endswith("/v1/"):
        return _get_interface_v1(int_name, depth, selector, **kwargs)
    else:   # Updated else for when version is v10.04
        return _get_interface(int_name, depth, selector, **kwargs)


def _get_interface_v1(int_name, depth=0, selector=None, **kwargs):
    """
    Perform a GET call to retrieve data for an Interface table entry

    :param int_name: Alphanumeric name of the interface
    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', or 'statistics'.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing data for Interface entry
    """
    int_name_percents = common_ops._replace_special_characters(int_name)

    if selector not in ['configuration', 'status', 'statistics', None]:
        raise Exception("ERROR: Selector should be 'configuration', 'status', or 'statistics'")

    target_url = kwargs["url"] + "system/interfaces/%s?" % int_name_percents
    payload = {
        "depth": depth,
        "selector": selector
    }
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=3)

    result = []
    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting Interface table entry '%s' failed with status code %d: %s"
              % (int_name, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting Interface table entry '%s' succeeded" % int_name)
        result = response.json()
    return result


def _get_interface(int_name, depth=0, selector=None, **kwargs):
    """
    Perform a GET call to retrieve data for an Interface table entry

    :param int_name: Alphanumeric name of the interface
    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', 'statistics' or 'writable'.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing data for Interface entry
    """
    int_name_percents = common_ops._replace_special_characters(int_name)

    if selector not in ['configuration', 'status', 'statistics', 'writable', None]:
        raise Exception("ERROR: Selector should be 'configuration', 'status', 'statistics', or 'writable'")

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    payload = {
        "depth": depth,
        "selector": selector
    }
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=3)
    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting Interface table entry '%s' failed with status code %d: %s"
              % (int_name, response.status_code, response.text))
        result = {}
    else:
        logging.info("SUCCESS: Getting Interface table entry '%s' succeeded" % int_name)
        result = response.json()

    return result


def get_all_interfaces(**kwargs):
    """
    Perform a GET call to get a list (or dictionary) of all entries in the Interface table

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List/dict of all Interfaces in the table
    """
    target_url = kwargs["url"] + "system/interfaces"

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list/dict of all Interface table entries failed with status code %d: %s"
              % (response.status_code, response.text))
        interface_list = []
    else:
        logging.info("SUCCESS: Getting list/dict of all Interface table entries succeeded")
        interface_list = response.json()

    return interface_list


def get_all_interface_names(**kwargs):
    """
    Perform a GET call to get a list of all of the names for each interface in the Interface table

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of all Interface names in the table
    """
    target_url = kwargs["url"] + "system/interfaces"

    response = kwargs["s"].get(target_url, verify=False)
    interface_list = []

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list of Interface names failed with status code %d: %s"
              % (response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting list of Interface names succeeded")
        uri_list = response.json()
        if kwargs["url"].endswith("/v1/"):
            for interface_uri in uri_list:
                interface_name = interface_uri[(interface_uri.rfind('/')+1):]  # Takes string after last '/'
                if interface_name != "bridge_normal":  # Ignore bridge_normal interface
                    interface_list.append(common_ops._replace_percents(interface_name))
        else:  # Updated else for when version is v10.04
            for interface_key in uri_list:
                interface_list.append(interface_key)
    return interface_list


def get_ipv6_addresses(int_name, depth=0, **kwargs):
    """
    Perform a GET call to retrieve the list of IPv6 addresses for an Interface table entry

    :param int_name: Alphanumeric name of the interface
    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of all ipv6 addresses for the Interface entry
    """
    int_name_percents = common_ops._replace_special_characters(int_name)

    if kwargs["url"].endswith("/v1/"):
        target_url = kwargs["url"] + "system/ports/%s/ip6_addresses" % int_name_percents
        logport = "Port"
    else:  # Updated else for when version is v10.04
        target_url = kwargs["url"] + "system/interfaces/%s/ip6_addresses" % int_name_percents
        logport = "Interface"

    payload = {
        "depth": depth
    }
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=3)
    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting IPv6 list for %s table entry '%s' failed with status code %d: %s"
              % (logport, int_name, response.status_code, response.text))
        result = []
    else:
        logging.info("SUCCESS: Getting IPv6 list for %s table entry '%s' succeeded" % (logport, int_name))
        result = response.json()

    return result


def add_vlan_interface(vlan_int_name, vlan_port_name, vlan_id, ipv4, vrf_name, vlan_port_desc, int_type="vlan",
                       user_config=None, **kwargs):
    """
    Perform a POST call to add Interface table entry for a VLAN.

    :param vlan_int_name: Alphanumeric name for the VLAN interface
    :param vlan_port_name: Alphanumeric Port name to associate with the interface
    :param vlan_id: Numeric ID of VLAN
    :param ipv4: Optional IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param vrf_name: VRF to attach the SVI to. Defaults to "default" if not specified
    :param vlan_port_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param int_type: Type of interface; generally should be "vlan" for SVI's.
        As such, defaults to "internal" if not specified.
    :param user_config: User configuration to apply to interface. Defaults to {"admin": "up"} if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _add_vlan_interface_v1(vlan_int_name, vlan_port_name, int_type, user_config, **kwargs)
    else:  # Updated else for when version is v10.04
        return _add_vlan_interface(vlan_int_name, vlan_id, ipv4, vrf_name, vlan_port_desc, int_type, user_config, **kwargs)


def _add_vlan_interface_v1(vlan_int_name, vlan_port_name, int_type="vlan", user_config=None, **kwargs):
    """
    Perform a POST call to add Interface table entry for a VLAN.

    :param vlan_int_name: Alphanumeric name for the VLAN interface
    :param vlan_port_name: Alphanumeric Port name to associate with the interface
    :param int_type: Type of interface; generally should be "vlan" for SVI's.
        As such, defaults to "internal" if not specified.
    :param user_config: User configuration to apply to interface. Defaults to {"admin": "up"} if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    ints_list = get_all_interfaces(**kwargs)

    if "/rest/v1/system/interfaces/%s" % vlan_int_name not in ints_list:
        if user_config is None:
            # optional argument can't default to a dictionary type,
            # so make it None and change it to the dictionary {"admin": "up"} if it was None
            user_config = {"admin": "up"}

        vlan_int_data = {"name": vlan_int_name,
                         "referenced_by": "/rest/v1/system/ports/%s" % vlan_port_name,
                         "type": int_type,  # API says: "vlan: generally represents SVI - L3 VLAN interfaces."
                         "user_config": user_config
                         }

        target_url = kwargs["url"] + "system/interfaces"

        post_data = json.dumps(vlan_int_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding Interface table entry '%s' for SVI failed with status code %d: %s"
                  % (vlan_int_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding Interface table entry '%s' for SVI succeeded" % vlan_int_name)
            return True
    else:
        logging.info("SUCCESS: No need to create Interface table entry '%s' for SVI since it already exists"
              % vlan_int_name)
        return True


def _add_vlan_interface(vlan_int_name, vlan_id=None, ipv4=None, vrf_name="default", vlan_port_desc=None,
                        int_type="vlan", user_config=None, **kwargs):
    """
    Perform a POST call to add Interface table entry for a VLAN.

    :param vlan_int_name: Alphanumeric name for the VLAN interface
    :param vlan_port_name: Alphanumeric Port name to associate with the interface
    :param int_type: Type of interface; generally should be "vlan" for SVI's.
        As such, defaults to "internal" if not specified.
    :param user_config: User configuration to apply to interface. Defaults to {"admin": "up"} if not speicifed.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    ints_dict = get_all_interfaces(**kwargs)

    if vlan_int_name not in ints_dict:
        if user_config is None:
            # optional argument can't default to a dictionary type,
            # so make it None and change it to the dictionary {"admin": "up"} if it was None
            user_config = {"admin": "up"}

        vlan_int_data = {"name": vlan_int_name,
                         "type": int_type,  # API says: "vlan: generally represents SVI - L3 VLAN interfaces."
                         "user_config": user_config,
                         "vrf": "/rest/v10.04/system/vrfs/%s" % vrf_name,
                         "vlan_tag": "/rest/v10.04/system/vlans/%s" % vlan_id
                         }

        if vlan_port_desc is not None:
            vlan_int_data['description'] = vlan_port_desc

        if ipv4 is not None:
            vlan_int_data['ip4_address'] = ipv4

        target_url = kwargs["url"] + "system/interfaces"

        post_data = json.dumps(vlan_int_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding Interface table entry '%s' for SVI failed with status code %d: %s"
                  % (vlan_int_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding Interface table entry '%s' for SVI succeeded" % vlan_int_name)
            return True
    else:
        logging.info("SUCCESS: No need to create Interface table entry '%s' for SVI since it already exists"
              % vlan_int_name)
        return True


def add_l2_interface(interface_name, interface_desc=None, interface_admin_state="up", **kwargs):
    """
    Perform a POST call to create an Interface table entry for physical L2 interface.

    :param interface_name: Alphanumeric Interface name
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port.add_l2_port(interface_name, interface_desc, interface_admin_state, **kwargs)
    else:   # Updated else for when version is v10.04
        return _add_l2_interface(interface_name, interface_desc, interface_admin_state, **kwargs)


def _add_l2_interface(interface_name, interface_desc=None, interface_admin_state="up", **kwargs):
    """
    Perform a PUT call to create an Interface table entry for physical L2 interface.
    :param interface_name: Alphanumeric Interface name
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    interface_name_percents = common_ops._replace_special_characters(interface_name)

    interface_data = {
        "admin": "up",
        "description": interface_desc,
        "routing": False,
        "user_config": {
            "admin": interface_admin_state
        },
    }

    target_url = kwargs["url"] + "system/interfaces/" + interface_name_percents
    post_data = json.dumps(interface_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=post_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Adding Interface table entry '%s' failed with status code %d: %s"
              % (interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Adding Interface table entry '%s' succeeded" % interface_name)
        return True


def add_l3_ipv4_interface(interface_name, ip_address=None, interface_desc=None, interface_admin_state="up",
                          vrf="default", **kwargs):
    """
    Perform a PUT or POST call to create an Interface table entry for a physical L3 Interface. If the Interface already
    exists, the function will enable routing on the Interface and update the IPv4 address if given.

    :param interface_name: Alphanumeric Interface name
    :param ip_address: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _add_l3_ipv4_interface_v1(interface_name, ip_address, interface_desc, interface_admin_state, vrf, **kwargs)
    else:   # Updated else for when version is v10.04
        return _add_l3_ipv4_interface(interface_name, ip_address, interface_desc, interface_admin_state, vrf, **kwargs)


def _add_l3_ipv4_interface_v1(interface_name, ip_address=None, interface_desc=None, interface_admin_state="up",
                              vrf="default", **kwargs):
    """
    Perform a PUT or POST call to create an Interface table entry for a physical L3 Interface. If the Interface already
    exists, the function will enable routing on the Interface and update the IPv4 address if given.

    :param interface_name: Alphanumeric Interface name
    :param ip_address: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    return port.add_l3_ipv4_port(interface_name, ip_address, interface_desc, interface_admin_state, vrf, **kwargs)


def _add_l3_ipv4_interface(interface_name, ip_address=None, interface_desc=None, interface_admin_state="up",
                           vrf="default", **kwargs):
    """
    Perform a PUT call to update an Interface table entry for a physical L3 Interface. If the Interface already
    exists, the function will enable routing on the Interface and update the IPv4 address if given.

    :param interface_name: Alphanumeric Interface name
    :param ip_address: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    interface_name_percents = common_ops._replace_special_characters(interface_name)

    interface_data = {
        "admin": interface_admin_state,
        "interfaces": ["/rest/v10.04/system/interfaces/%s" % interface_name_percents],
        "routing": True,
        "ip4_address": ip_address,
        "vrf": "/rest/v10.04/system/vrfs/%s" % vrf
    }

    if interface_desc is not None:
        interface_data['description'] = interface_desc

    target_url = kwargs["url"] + "system/interfaces/" + interface_name_percents
    put_data = json.dumps(interface_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Adding Interface table entry '%s' failed with status code %d: %s"
              % (interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Configuring Interface table entry '%s' succeeded" % interface_name)
        return True


def add_l3_ipv6_interface(interface_name, ip_address=None, interface_desc=None, interface_admin_state="up",
                          vrf="default", **kwargs):
    """
    Perform a PUT or POST call to create an Interface table entry for a physical L3 Interface. If the Interface already
    exists, the function will enable routing on the Interface and update the IPv6 address if given.

    :param interface_name: Alphanumeric Interface name
    :param ip_address: IPv6 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _add_l3_ipv6_interface_v1(interface_name, ip_address, interface_desc, interface_admin_state, vrf, **kwargs)
    else:   # Updated else for when version is v10.04
        return _add_l3_ipv6_interface(interface_name, ip_address, interface_desc, interface_admin_state, vrf, **kwargs)


def _add_l3_ipv6_interface_v1(interface_name, ip_address=None, interface_desc=None, interface_admin_state="up",
                              vrf="default", **kwargs):
    """
    Perform a PUT or POST call to create an Interface table entry for a physical L3 Interface. If the Interface already
    exists, the function will enable routing on the Interface and update the IPv6 address if given.

    :param interface_name: Alphanumeric Interface name
    :param ip_address: IPv6 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    return port.add_l3_ipv6_port(interface_name, ip_address, interface_desc, interface_admin_state, vrf, **kwargs)


def _add_l3_ipv6_interface(interface_name, ip_address=None, interface_desc=None, interface_admin_state="up",
                           vrf="default", **kwargs):
    """
    Perform a PUT call to update an Interface table entry for a physical L3 Interface, then a POST call to add an IPv6
    mapping. If the Interface already exists, the function will enable routing on the Interface and update the
    IPv6 address if given.

    :param interface_name: Alphanumeric Interface name
    :param ip_address: IPv6 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param interface_admin_state: Optional administratively-configured state of the interface.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    interface_name_percents = common_ops._replace_special_characters(interface_name)

    interface_data = {
        "admin": interface_admin_state,
        "interfaces": ["/rest/v10.04/system/interfaces/%s" % interface_name_percents],
        "routing": True,
        "vrf": {vrf: "/rest/v10.04/system/vrfs/%s" % vrf}
    }

    if interface_desc is not None:
        interface_data['description'] = interface_desc

    target_url = kwargs["url"] + "system/interfaces/" + interface_name_percents
    put_data = json.dumps(interface_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Initial configuration of L3 IPv6 Interface table entry '%s' failed with status code %d: %s"
              % (interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Initial configuration of L3 IPv6 Interface table entry '%s' succeeded" % interface_name)
        # IPv6 defaults
        ipv6_data = {
          "address": ip_address,
          "node_address": True,
          "origin": "configuration",
          "ra_prefix": True,
          "route_tag": 0,
          "type": "global-unicast"
        }

        target_url = kwargs["url"] + "system/interfaces/%s/ip6_addresses" % interface_name_percents
        post_data = json.dumps(ipv6_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)
        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Final configuration of L3 IPv6 Interface table entry '%s' failed with status code %d: %s"
                  % (interface_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Final configuration of L3 IPv6 Interface table entry '%s' succeeded" % interface_name)
            return True


def delete_ipv6_address(interface_name, ip, **kwargs):
    """
    Perform a DELETE call to remove an IPv6 address from an Interface.

    :param interface_name: Alphanumeric Interface name
    :param ip: IPv6 address assigned to the interface that will be removed.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port._delete_ipv6_address(interface_name, ip, **kwargs)
    else:   # Updated else for when version is v10.04
        return _delete_ipv6_address(interface_name, ip, **kwargs)


def _delete_ipv6_address(interface_name, ip, **kwargs):
    """
    Perform a DELETE call to remove an IPv6 address from an Interface.

    :param interface_name: Alphanumeric Interface name
    :param ip: IPv6 address assigned to the interface that will be removed.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if ip in get_ipv6_addresses(interface_name, **kwargs):
        interface_name_percents = common_ops._replace_special_characters(interface_name)
        ip_address = common_ops._replace_special_characters(ip)
        target_url = kwargs["url"] + "system/interfaces/%s/ip6_addresses/%s" % (interface_name_percents, ip_address)

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting IPv6 Address '%s' from Interface table entry '%s' failed with status code %d: %s"
                  % (ip, interface_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting IPv6 Address '%s' from Interface table entry '%s' succeeded"
                  % (ip, interface_name))
            return True
    else:
        logging.info("SUCCESS: No need to delete IPv6 Address '%s' from Interface table entry '%s' since it does not exist"
              % (ip, interface_name))
        return True


def create_loopback_interface(interface_name, vrf="default", ipv4=None, interface_desc=None, **kwargs):
    """
    Perform a PUT and/or POST call to create a Loopback Interface table entry for a logical L3 Interface. If the
    Loopback Interface already exists and an IPv4 address is given, the function will update the IPv4 address.

    :param interface_name: Alphanumeric Interface name
    :param vrf: VRF to attach the SVI to. Defaults to "default" if not specified
    :param ipv4: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _create_loopback_interface_v1(interface_name, vrf, ipv4, interface_desc, **kwargs)
    else:   # Updated else for when version is v10.04
        return _create_loopback_interface(interface_name, vrf, ipv4, interface_desc, **kwargs)


def _create_loopback_interface_v1(interface_name, vrf, ipv4=None, interface_desc=None, **kwargs):
    """
    Perform a PUT and/or POST call to create a Loopback Interface table entry for a logical L3 Interface. If the
    Loopback Interface already exists and an IPv4 address is given, the function will update the IPv4 address.

    :param interface_name: Alphanumeric Interface name
    :param vrf: VRF to attach the SVI to. Defaults to "default" if not specified
    :param ipv4: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    port.create_loopback_port(interface_name, vrf, ipv4, interface_desc, **kwargs)


def _create_loopback_interface(interface_name, vrf, ipv4=None, interface_desc=None, **kwargs):
    """
    Perform a POST call to create a Loopback Interface table entry for a logical L3 Interface. If the
    Loopback Interface already exists and an IPv4 address is given, the function will update the IPv4 address.

    :param interface_name: Alphanumeric Interface name
    :param vrf: VRF to attach the SVI to. Defaults to "default" if not specified
    :param ipv4: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param interface_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    interface_name_percents = common_ops._replace_special_characters(interface_name)

    interface_data = {
        "name": interface_name,
        "type": "loopback",
        "user_config": {
            "admin": "up"
        },
        "ospf_if_type": "ospf_iftype_loopback",
        "vrf": "/rest/v10.04/system/vrfs/%s" % vrf
    }

    if ipv4 is not None:
        interface_data['ip4_address'] = ipv4

    if interface_desc is not None:
        interface_data['description'] = interface_desc

    target_url = kwargs["url"] + "system/interfaces"
    post_data = json.dumps(interface_data, sort_keys=True, indent=4)

    response = kwargs["s"].post(target_url, data=post_data, verify=False)

    if not common_ops._response_ok(response, "POST"):
        logging.warning("FAIL: Adding Interface table entry '%s' failed with status code %d: %s"
              % (interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Configuring Interface table entry '%s' succeeded" % interface_name)
        return True


def _create_vxlan_interface(interface_name, source_ipv4=None, port_desc=None, dest_udp_port=4789, **kwargs):
    """
    Perform POST call to create a VXLAN table entry for a logical L3 Interface. If the
    VXLAN Interface already exists and an IPv4 address is given, the function will update the IPv4 address.

    :param interface_name: Alphanumeric Interface name
    :param source_ipv4: Optional source IPv4 address to assign to the VXLAN interface. Defaults to nothing if not specified.
    :param port_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param dest_udp_port: Optional Destination UDP Port that the VXLAN will use.  Default is set to 4789
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    interfaces_dict = get_all_interfaces(**kwargs)

    if interface_name not in interfaces_dict:
        interface_data = {
            "name": interface_name,
            "options": {
                "local_ip": source_ipv4,
                "vxlan_dest_udp_port": str(dest_udp_port)
            },
            "type": "vxlan",
            "user_config": {
                "admin": "up"
            },

            "admin": "up",
            "routing": False
        }

        if port_desc is not None:
            interface_data['description'] = port_desc

        interface_url = kwargs["url"] + "system/interfaces"
        post_data = json.dumps(interface_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(interface_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding VXLAN Interface table entry '%s' failed with status code %d: %s"
                  % (interface_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding VXLAN Interface table entry '%s' succeeded" % interface_name)
            return True
    else:
        return update_interface_ipv4(interface_name, source_ipv4, **kwargs)


def update_interface_ipv4(interface_name, ipv4, interface_admin_state, vrf, **kwargs):
    """
    Perform GET and PUT calls to update an L3 interface's ipv4 address

    :param interface_name: Alphanumeric name of the Port
    :param ipv4: IPv4 address to associate with the VLAN Port
    :param interface_admin_state: Administratively-configured state of the port.
    :param vrf: Name of the VRF to which the Port belongs.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    interface_name_percents = common_ops._replace_special_characters(interface_name)
    interface_data = get_interface(interface_name, depth=1, selector="writable", **kwargs)

    interface_data['ip4_address'] = ipv4
    interface_data['routing'] = True
    interface_data['admin'] = interface_admin_state
    interface_data['vrf'] = "/rest/v10.04/system/vrfs/%s" % vrf

    target_url = kwargs["url"] + "system/interfaces/%s" % interface_name_percents
    put_data = json.dumps(interface_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating IPv4 addresses for Port '%s' to '%s' failed with status code %d: %s"
              % (interface_name, repr(ipv4), response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating IPv4 addresses for Port '%s' to '%s' succeeded"
              % (interface_name, repr(ipv4)))
        return True


def update_port_ipv6(interface_name, ipv6, addr_type="global-unicast", **kwargs):
    """
    Perform a POST call to update an L3 interface's ipv6 address

    :param interface_name: Alphanumeric name of the Port
    :param ipv6: IPv6 address to associate with the VLAN Port
    :param addr_type: Type of IPv6 address. Defaults to "global-unicast" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    ipv6_data = {"address": ipv6,
                 "type": addr_type}

    target_url = kwargs["url"] + "system/interfaces/%s/ip6_addresses" % interface_name
    post_data = json.dumps(ipv6_data, sort_keys=True, indent=4)

    response = kwargs["s"].post(target_url, data=post_data, verify=False)

    if not common_ops._response_ok(response, "POST"):
        logging.warning("FAIL: Updating IPv6 address for Port '%s' to '%s' failed with status code %d: %s"
              % (interface_name, ipv6, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating IPv6 address for Port '%s' to '%s' succeeded"
              % (interface_name, ipv6))
        return True


def enable_disable_interface(int_name, state="up", **kwargs):
    """
    Perform GET and PUT calls to either enable or disable the interface by setting Interface's admin_state to
        "up" or "down"

    :param int_name: Alphanumeric name of the interface
    :param state: State to set the interface to
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _enable_disable_interface_v1(int_name, state, **kwargs)
    else:  # Updated else for when version is v10.04
        return _enable_disable_interface(int_name, state, **kwargs)


def _enable_disable_interface_v1(int_name, state="up", **kwargs):
    """
    Perform GET and PUT calls to either enable or disable the interface by setting Interface's admin_state to
        "up" or "down"

    :param int_name: Alphanumeric name of the interface
    :param state: State to set the interface to
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if state not in ['up', 'down']:
        raise Exception("Administratively-configured state of interface should be 'up' or 'down'")

    int_name_percents = common_ops._replace_special_characters(int_name)

    interface_list = get_all_interfaces(**kwargs)

    if "/rest/v1/system/interfaces/%s" % int_name_percents in interface_list:
        int_data = get_interface(int_name, 0, "configuration", **kwargs)
        int_data['user_config'] = {"admin": state}

        target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
        put_data = json.dumps(int_data, sort_keys=True, indent=4)

        response = kwargs["s"].put(target_url, data=put_data, verify=False)

        if not common_ops._response_ok(response, "PUT"):
            logging.warning("FAIL: Updating Interface '%s' with admin-configured state '%s' "
                  "failed with status code %d: %s" % (int_name, state, response.status_code, response.text))
            success = False
        else:
            logging.info("SUCCESS: Updating Interface '%s' with admin-configured state '%s' "
                  "succeeded" % (int_name, state))
            success = True
        port._enable_disable_port(int_name, state, **kwargs)
        return success
    else:
        logging.warning("FAIL: Unable to update Interface '%s' because operation could not find interface" % int_name)
        return False


def _enable_disable_interface(int_name, state="up", **kwargs):
    """
    Perform GET and PUT calls to either enable or disable the interface by setting Interface's admin_state to
        "up" or "down"

    :param int_name: Alphanumeric name of the interface
    :param state: State to set the interface to
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if state not in ['up', 'down']:
        raise Exception("Administratively-configured state of interface should be 'up' or 'down'")

    int_name_percents = common_ops._replace_special_characters(int_name)

    int_data = get_interface(int_name, 1, "writable", **kwargs)
    int_data['user_config'] = {"admin": state}

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating Interface '%s' with admin-configured state '%s' "
              "failed with status code %d: %s" % (int_name, state, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating Interface '%s' with admin-configured state '%s' "
              "succeeded" % (int_name, state))
        return True


def delete_interface(interface_name, **kwargs):
    """
    Perform a DELETE call to either the Interface Table or Port Table to delete an interface

    :param interface_name: Name of interface's reference entry in Interface table
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _delete_interface_v1(interface_name, **kwargs)
    else:  # Updated else for when version is v10.04
        return _delete_interface(interface_name, **kwargs)


def _delete_interface_v1(interface_name, **kwargs):
    """
    Perform DELETE call to Port Table to delete an interface

    Note: Interface API does not have delete methods.
    To delete an Interface, you remove its reference port.

    :param name: Name of interface's reference entry in Port table
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    return port.delete_port(interface_name, **kwargs)


def _delete_interface(name, **kwargs):
    """
    Perform DELETE call to Interface table to delete an interface
    :param name: Name of interface
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    ints_dict = get_all_interfaces(**kwargs)

    if name in ints_dict:

        target_url = kwargs["url"] + "system/interfaces/%s" % name

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting Interface table entry '%s' failed with status code %d: %s"
                  % (name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting Interface table entry '%s' succeeded" % name)
            return True
    else:
        logging.info("SUCCESS: No need to delete Interface table entry '%s' because it doesn't exist"
              % name)
        return True


def delete_l2_interface(interface_name, **kwargs):
    """
    Perform either a PUT call to the Interface Table or DELETE call to Port Table to delete an interface
    If trying to re-initialize an L2 interface, use the function initialize_l2_interface()

    :param interface_name: Name of interface's reference entry in Interface table
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _delete_l2_interface_v1(interface_name, **kwargs)
    else:  # Updated else for when version is v10.04
        return _delete_l2_interface(interface_name, **kwargs)


def _delete_l2_interface_v1(interface_name, **kwargs):
    """
    Perform DELETE call to Port Table to delete an L2 interface

    Note: Interface API does not have delete methods.
    To delete an Interface, you remove its reference port.

    :param name: Name of interface's reference entry in Port table
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    return port.delete_port(interface_name, **kwargs)


def _delete_l2_interface(interface_name, **kwargs):
    """
    Perform a PUT call to the Interface Table to reset an interface to it's default values

    :param interface_name: Name of interface's reference entry in Interface table
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    interface_name_percents = common_ops._replace_special_characters(interface_name)
    target_url = kwargs["url"] + "system/interfaces/%s" % interface_name_percents

    interface_data = {}
    interface_data = json.dumps(interface_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=interface_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Deleting Interface '%s' failed with status code %d: %s"
                        % (interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Deleting Interface '%s' succeeded" % interface_name)
        return True


def _port_set_vlan_mode(l2_port_name, vlan_mode, **kwargs):
    """
    Perform GET and PUT calls to set an L2 interface's VLAN mode (native-tagged, native-untagged, or access)

    :param l2_port_name: L2 interface's Interface table entry name
    :param vlan_mode: A string, either 'native-tagged', 'native-untagged', or 'access', specifying the desired VLAN
        mode
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if vlan_mode not in ['native-tagged', 'native-untagged', 'access']:
        raise Exception("ERROR: VLAN mode should be 'native-tagged', 'native-untagged', or 'access'")

    l2_port_name_percents = common_ops._replace_special_characters(l2_port_name)
    int_data = get_interface(l2_port_name_percents, depth=1, selector="writable", **kwargs)

    int_data['vlan_mode'] = vlan_mode
    int_data['routing'] = False

    target_url = kwargs["url"] + "system/interfaces/%s" % l2_port_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Setting port '%s' VLAN mode to '%s' failed with status code %d: %s"
              % (l2_port_name, vlan_mode, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Setting port '%s' VLAN mode to '%s' succeeded" % (l2_port_name, vlan_mode))
        return True


def _port_set_untagged_vlan(l2_port_name, vlan_id, **kwargs):
    """
    Perform GET and PUT/POST calls to set a VLAN on an access port

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_id: Numeric ID of VLAN to set on access port
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    l2_port_name_percents = common_ops._replace_special_characters(l2_port_name)

    int_data = get_interface(l2_port_name_percents, depth=1, selector="writable", **kwargs)

    int_data['vlan_mode'] = "access"
    int_data['vlan_tag'] = "/rest/v10.04/system/vlans/%s" % vlan_id
    int_data['routing'] = False

    target_url = kwargs["url"] + "system/interfaces/%s" % l2_port_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Setting Port '%s' access VLAN to VLAN ID '%d' failed with status code %d: %s"
              % (l2_port_name, vlan_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Setting Port '%s' access VLAN to VLAN ID '%d' succeeded"
              % (l2_port_name, vlan_id))
        return True


def _port_add_vlan_trunks(l2_port_name, vlan_trunk_ids={}, **kwargs):
    """
    Perform GET and PUT/POST calls to add specified VLANs to a trunk port. By default, this will also set the port to
    have 'no routing' and if there is not a native VLAN, will set the native VLAN to VLAN 1.

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_trunk_ids: Dictionary of VLANs to specify as allowed on the trunk port.  If empty, the interface will
        allow all VLANs on the trunk.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    l2_port_name_percents = common_ops._replace_special_characters(l2_port_name)

    trunk_list = {}
    for x in vlan_trunk_ids:
        x_keys = {str(x): "/rest/v10.04/system/vlans/%d" % x}
        trunk_list.update(x_keys)

    port_data = get_interface(l2_port_name, depth=1, selector="writable", **kwargs)

    if not port_data['vlan_tag']:
        port_data['vlan_tag'] = "/rest/v10.04/system/vlans/1"
    else:
        # Convert the dictionary to a URI string
        port_data['vlan_tag'] = common_ops._dictionary_to_string(port_data['vlan_tag'])

    if not port_data['vlan_mode']:
        port_data['vlan_mode'] = "native-untagged"
    port_data['routing'] = False

    if not trunk_list:
        port_data['vlan_trunks'] = []
    else:
        for key in trunk_list:
            if key not in port_data['vlan_trunks']:
                port_data['vlan_trunks'].append("/rest/v10.04/system/vlans/%s" % key)

    target_url = kwargs["url"] + "system/interfaces/%s" % l2_port_name_percents
    put_data = json.dumps(port_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Adding VLANs '%s' to Port '%s' trunk failed with status code %d: %s"
              % (vlan_trunk_ids, l2_port_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Adding VLANs '%s' to Port '%s' trunk succeeded"
              % (vlan_trunk_ids, l2_port_name))
        return True


def _port_set_native_vlan(l2_port_name, vlan_id, tagged=True, **kwargs):
    """
    Perform GET and PUT/POST calls to set a VLAN to be the native VLAN on the trunk. Also gives the option to set
    the VLAN as tagged.

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_id: Numeric ID of VLAN to add to trunk port
    :param tagged: Boolean to determine if the native VLAN will be set as the tagged VLAN.  If False, the VLAN
        will be set as the native untagged VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if tagged:
        vlan_mode = "native-tagged"
    else:
        vlan_mode = "native-untagged"

    l2_port_name_percents = common_ops._replace_special_characters(l2_port_name)
    vlan_uri = "/rest/v10.04/system/vlans/%d" % vlan_id
    vlan_key = {str(vlan_id): vlan_uri}
    port_data = get_interface(l2_port_name_percents, depth=1, selector="writable", **kwargs)

    port_data['vlan_tag'] = vlan_uri
    port_data['routing'] = False
    port_data['vlan_mode'] = vlan_mode

    if (port_data['vlan_trunks']) and (vlan_key not in port_data['vlan_trunks']):
        port_data['vlan_trunks'].update(vlan_key)

    target_url = kwargs["url"] + "system/interfaces/%s" % l2_port_name_percents
    put_data = json.dumps(port_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Setting native VLAN ID '%d' to Port '%s' failed with status code %d: %s"
              % (vlan_id, l2_port_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Setting native VLAN ID '%d' to Port '%s' succeeded"
              % (vlan_id, l2_port_name))
        return True


def _delete_vlan_port(l2_port_name, vlan_id, **kwargs):
    """
    Perform GET and PUT calls to remove a VLAN from a trunk port

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_id: Numeric ID of VLAN to remove from trunk port
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    l2_port_name_percents = common_ops._replace_special_characters(l2_port_name)

    port_data = get_interface(l2_port_name, depth=1, selector="writable", **kwargs)

    if str(vlan_id) in port_data['vlan_trunks']:
        # remove vlan from 'vlan_trunks'
        port_data['vlan_trunks'].pop(str(vlan_id))

    target_url = kwargs["url"] + "system/interface/%s" % l2_port_name_percents
    put_data = json.dumps(port_data, sort_keys=True, indent=4)
    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Removing VLAN ID '%d' from Port '%s' trunk failed with status code %d: %s"
              % (vlan_id, l2_port_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Removing VLAN ID '%d' from Port '%s' trunk succeeded"
              % (vlan_id, l2_port_name))
        return True


def add_port_to_lag(int_name, lag_id, **kwargs):
    """
    Perform GET and PUT calls to configure a Port as a LAG member, and also enable the port. For v1,
    also perform DELETE call to remove the Port table entry for the port.

    :param int_name: Alphanumeric name of the interface
    :param lag_id: Numeric ID of the LAG to which the port is to be added
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _add_port_to_lag_v1(int_name, lag_id, **kwargs)
    else:  # Updated else for when version is v10.04
        return _add_port_to_lag(int_name, lag_id, **kwargs)


def _add_port_to_lag_v1(int_name, lag_id, **kwargs):
    """
    Perform GET and PUT calls to configure a Port as a LAG member, and also enable the port.
    Also perform DELETE call to remove the Port table entry for the port.

    :param int_name: Alphanumeric name of the interface
    :param lag_id: Numeric ID of the LAG to which the port is to be added
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    int_name_percents = common_ops._replace_special_characters(int_name)

    int_data = get_interface(int_name, 0, "configuration", **kwargs)

    int_data['user_config'] = {"admin": "up"}
    int_data['other_config']['lacp-aggregation-key'] = lag_id

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Adding Interface '%s' to LAG '%d' "
              "failed with status code %d: %s" % (int_name, lag_id, response.status_code, response.text))
        success = False
    else:
        logging.info("SUCCESS: Adding Interface '%s' to LAG '%d' "
              "succeeded" % (int_name, lag_id))
        success = True
    # Delete Port Table entry for the port
    return success and port.delete_port(int_name_percents, **kwargs)


def _add_port_to_lag(int_name, lag_id, **kwargs):
    """
    Perform GET and PUT calls to configure a Port as a LAG member, and also enable the port

    :param int_name: Alphanumeric name of the interface
    :param lag_id: Numeric ID of the LAG to which the port is to be added
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    int_name_percents = common_ops._replace_special_characters(int_name)

    int_data = get_interface(int_name, 1, "writable", **kwargs)

    int_data['user_config'] = {"admin": "up"}
    int_data['other_config']['lacp-aggregation-key'] = lag_id

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Adding Interface '%s' to LAG '%d' "
              "failed with status code %d: %s" % (int_name, lag_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Adding Interface '%s' to LAG '%d' "
              "succeeded" % (int_name, lag_id))
        return True


def remove_port_from_lag(int_name, lag_id, **kwargs):
    """
    Perform GET and PUT calls to configure a Port as a LAG member, and also disable the port

    :param int_name: Alphanumeric name of the interface
    :param lag_id: Numeric ID of the LAG to which the port is to be added
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _remove_port_from_lag_v1(int_name, lag_id, **kwargs)
    else:  # Updated else for when version is v10.04
        return _remove_port_from_lag(int_name, lag_id, **kwargs)


def _remove_port_from_lag_v1(int_name, lag_id, **kwargs):
    """
    Perform GET and PUT calls to remove a Port from a LAG, and also disable the port

    :param int_name: Alphanumeric name of the interface
    :param lag_id: Numeric ID of the LAG to which the port is to be added
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    # Create Port Table entry for the port
    add_l2_interface(int_name, **kwargs)

    int_name_percents = common_ops._replace_special_characters(int_name)

    int_data = get_interface(int_name, 0, "configuration", **kwargs)

    int_data['user_config'] = {"admin": "down"}
    int_data['other_config'].pop('lacp-aggregation-key', None)

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Removing interface '%s' from LAG '%d' "
              "failed with status code %d: %s" % (int_name, lag_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Removing interface '%s' from LAG '%d' "
              "succeeded" % (int_name, lag_id))
        return True


def _remove_port_from_lag(int_name, lag_id, **kwargs):
    """
    Perform GET and PUT calls to remove a Port from a LAG, and also disable the port

    :param int_name: Alphanumeric name of the interface
    :param lag_id: Numeric ID of the LAG to which the port is to be added
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    int_name_percents = common_ops._replace_special_characters(int_name)

    int_data = get_interface(int_name, 1, "writable", **kwargs)

    int_data['user_config'] = {"admin": "down"}
    int_data['other_config'].pop('lacp-aggregation-key', None)

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Removing interface '%s' from LAG '%d' "
              "failed with status code %d: %s" % (int_name, lag_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Removing interface '%s' from LAG '%d' "
              "succeeded" % (int_name, lag_id))
        return True


def _clear_interface_acl(interface_name, acl_type, **kwargs):
    """
    Perform GET and PUT calls to clear an interface's ACL

    :param port_name: Alphanumeric name of the Port
    :param acl_type: Type of ACL: options are 'aclv4_out', 'aclv4_in', 'aclv6_in', or 'aclv6_out'
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if acl_type not in ['aclv4_out', 'aclv4_in', 'aclv6_in', 'aclv6_out']:
        raise Exception("ERROR: acl_type should be 'aclv4_out', 'aclv4_in', 'aclv6_in', or 'aclv6_out'")

    int_name_percents = common_ops._replace_special_characters(interface_name)

    interface_data = get_interface(interface_name, depth=1, selector="writable", **kwargs)

    if interface_name.startswith('lag'):
        if interface_data['interfaces']:
            interface_data['interfaces'] = common_ops._dictionary_to_list_values(interface_data['interfaces'])

    cfg_type = acl_type + '_cfg'
    cfg_version = acl_type + '_cfg_version'

    interface_data.pop(cfg_type, None)
    interface_data.pop(cfg_version, None)

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(interface_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Clearing %s ACL on Interface '%s' failed with status code %d: %s"
              % (cfg_type, interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Clearing %s ACL on Interface '%s' succeeded"
              % (cfg_type, interface_name))
        return True


def initialize_interface_entry(int_name, **kwargs):
    """
    Perform a PUT call on the interface to initialize it to it's default state.

    :param int_name: Alphanumeric name of the system interface
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    int_name_percents = common_ops._replace_special_characters(int_name)
    int_data = {}
    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Initializing interface '%s' failed with status code %d: %s"
                        % (int_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Initializing interface '%s' succeeded" % int_name)
        success = True
        # Remove all IPv6 entries for this interface
        ipv6_list = get_ipv6_addresses(int_name, **kwargs)
        if ipv6_list:
            for ipv6_address in ipv6_list:
                success = success and delete_ipv6_address(int_name, ipv6_address, **kwargs)
        return success


def initialize_interface(interface_name, **kwargs):
    """
    Perform a PUT call to the Interface Table or Port Table to initialize an interface to factory settings

    :param interface_name: Name of interface's reference entry in Interface table
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port.initialize_port_entry(interface_name, **kwargs)
    else:  # Updated else for when version is v10.04
        return initialize_interface_entry(interface_name, **kwargs)


# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops

import json
import logging
import base64
import os


def get_all_nae_scripts(params={}, **kwargs):
    """
    Perform a GET call to get a list or dictionary of all the Network Analytics Engine scripts on the device.

    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List (v1) or Dictionary (v10.04 or empty) containing all of the NAE scripts on the device
    """
    target_url = kwargs["url"] + "system/nae_scripts"
    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting dictionary of all NAE scripts on device failed with status code %d: %s"
              % (response.status_code, response.text))
        all_nae_scripts = {}
    else:
        if kwargs["url"].endswith("/v1/"):
            logging.info("SUCCESS: Getting list of all NAE scripts on device succeeded")
        else:
            # Else logic designed for v10.04 and later
            logging.info("SUCCESS: Getting dictionary of all NAE scripts on device succeeded")
        all_nae_scripts = response.json()

    return all_nae_scripts


def get_nae_script(nae_script, params={}, **kwargs):
    """
    Perform a GET call to get the details of a specific Network Analytics Engine script on the device.

    :param nae_script: String of name of the script that the function will return
    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing the details of the specified NAE script on the device
    """
    target_url = kwargs["url"] + "system/nae_scripts/" + nae_script
    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting details of NAE script %s on device failed with status code %d: %s"
              % (nae_script, response.status_code, response.text))
        nae_script_details = {}
    else:
        logging.info("SUCCESS: Getting details of NAE script %s on device succeeded" % nae_script)
        nae_script_details = response.json()

    return nae_script_details


def get_nae_script_code(nae_script, params={}, **kwargs):
    """
    Perform a GET call to get the decoded Network Analytics Engine script from the device.

    :param nae_script: String of name of the script that the function will return
    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: String of NAE script python code, decoded from base64
    """
    target_url = kwargs["url"] + "system/nae_scripts/" + nae_script
    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting NAE script %s python code on device failed with status code %d: %s"
                        % (nae_script, response.status_code, response.text))
        nae_script_code = ""
    else:
        logging.info("SUCCESS: Getting NAE script %s python code on device succeeded" % nae_script)
        nae_script_details = response.json()
        nae_script_code = base64.b64decode(nae_script_details['script'])

    return nae_script_code


def load_nae_script(script_name, script, is_base64=True, **kwargs):
    """
    Perform a POST call to upload a Network Analytics Engine script to the device. This function passes the script
    code as a parameter. By default, is_base64 is True and the script expects the base64 encoded text for the script.
    If is_base64 is set to false, the function will expect a block of python code for the NAE script.

    :param script_name: Alphanumeric String of the name of the script
    :param script: the base64 text or python code of the NAE script
    :param is_base64: Boolean to determine if the script parameter is already encoded in base64. If the set to True,
        the function will take in the script as is.  If set to False, the function will encode the script to base64.
        By default, this is True.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _load_nae_script_v1(script_name, script, is_base64, **kwargs)
    else:  # Updated else for when version is v10.04
        return _load_nae_script(script_name, script, is_base64, **kwargs)


def _load_nae_script_v1(script_name, script, is_base64, **kwargs):
    """
    Perform a POST call to upload a Network Analytics Engine script to the device. This function passes the script
    code as a parameter. By default, is_base64 is True and the script expects the base64 encoded text for the script.
    If is_base64 is set to false, the function will expect a block of python code for the NAE script.

    :param script_name: Alphanumeric String of the name of the script
    :param script: the base64 text or python code of the NAE script
    :param is_base64: Boolean to determine if the script parameter is already encoded in base64. If the set to True,
        the function will take in the script as is.  If set to False, the function will encode the script to base64.
        By default, this is True.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    script_list = get_all_nae_scripts(**kwargs)

    if "/rest/v1/system/nae_scripts/%s" % script_name not in script_list:
        script_data = {
            "name": script_name
            }

        if is_base64:
            script_data["script"] = script
        else:
            script_data["script"] = base64.b64encode(script)

        target_url = kwargs["url"] + "system/nae_scripts"
        post_data = json.dumps(script_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Uploading NAE script named '%s' failed with status code %d: %s"
                  % (script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Uploading NAE script named '%s' succeeded" % script_name)
            return True
    else:
        logging.info("SUCCESS: Upload not needed; NAE Script named '%s' already exists on the system" % script_name)
        return True


def _load_nae_script(script_name, script, is_base64, **kwargs):
    """
    Perform a POST call to upload a Network Analytics Engine script to the device. This function passes the script
    code as a parameter. By default, is_base64 is True and the script expects the base64 encoded text for the script.
    If is_base64 is set to false, the function will expect a block of python code for the NAE script.

    :param script_name: Alphanumeric String of the name of the script
    :param script: the base64 text or python code of the NAE script
    :param is_base64: Boolean to determine if the script parameter is already encoded in base64. If the set to True,
        the function will take in the script as is.  If set to False, the function will encode the script to base64.
        By default, this is True.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    script_dict = get_all_nae_scripts(**kwargs)

    if script_name not in script_dict:
        script_data = {
            "expert_only": False,
            "name": script_name
            }

        if is_base64:
            script_data["script"] = script
        else:
            script_data["script"] = base64.b64encode(script)

        target_url = kwargs["url"] + "system/nae_scripts"
        post_data = json.dumps(script_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Uploading NAE script named '%s' failed with status code %d: %s"
                  % (script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Uploading NAE script named '%s' succeeded" % script_name)
            return True
    else:
        logging.info("SUCCESS: Upload not needed; NAE Script named '%s' already exists on the system" % script_name)
        return True


def load_nae_script_from_file(script_name, path_and_filename, is_base64=False, **kwargs):
    """
    Retrieves a specified NAE script and uploads it to the device. This function will read the specified python file
    then perform a POST call to upload the script to the device. By default, is_base64 is False and the function expects
    to encoded the text in base64 prior to uploading.
    If is_base64 is set to True, the function will attempt to encode the python code prior to uploading.

    :param script_name: Alphanumeric String of the name of the script
    :param path_and_filename: Alphanumeric String of the path and filename to the NAE python script
    :param is_base64: Boolean to determine if the script parameter is already encoded in base64. If the set to True,
        the function will take in the script as is.  If set to False, the function will encode the script to base64.
        By default, this is True.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    with open(path_and_filename, 'rb') as py_file:
        py_file_data = py_file.read()

    if not is_base64:
        output_script = base64.b64encode(py_file_data)
        output_script = output_script.decode('utf-8')

    else:
        output_script = py_file_data

    return load_nae_script(script_name, output_script, is_base64=True, **kwargs)


def delete_nae_script(script_name, **kwargs):
    """
    Perform a DELETE call to remove a Network Analytics Engine script from the system.

    *Note that by removing a script, all associated agents of the script will be removed as well.
    :param script_name: Alphanumeric String of the name of the script
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        script_check = "/rest/v1/system/nae_scripts/%s" % script_name
    else:  # Updated else for when version is v10.04
        script_check = script_name

    all_scripts = get_all_nae_scripts(**kwargs)

    if script_check not in all_scripts:
        logging.info("SUCCESS: NAE Script deletion not needed; NAE Script named '%s' does NOT exist on the system"
                     % script_name)
        return True

    else:
        target_url = kwargs["url"] + "system/nae_scripts/%s" % script_name
        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting NAE script named '%s' failed with status code %d: %s"
                  % (script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting NAE script named '%s' succeeded" % script_name)
            return True


def get_all_nae_agents_of_script(script_name, params={}, **kwargs):
    """
    Perform a GET call to get a list or dictionary of Network Analytics Engine agents for the specified NAE script.

    :param script_name: Alphanumeric String of the name of the script
    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List (v1) or Dictionary (v10.04 or empty) containing all of the existing agents for the specified script.
    """
    target_url = kwargs["url"] + "system/nae_scripts/%s/nae_agents" % script_name
    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list/dictionary of all NAE agents for NAE script named %s failed with "
                        "status code %d: %s" % (script_name, response.status_code, response.text))
        all_nae_agents = {}
    else:
        if kwargs["url"].endswith("/v1/"):
            logging.info("SUCCESS: Getting list of all NAE agents for script named %s on device succeeded"
                         % script_name)
        else:
            # Else logic designed for v10.04 and later
            logging.info("SUCCESS: Getting dictionary of all NAE agents for script named %s on device succeeded"
                         % script_name)
        all_nae_agents = response.json()

    return all_nae_agents


def get_nae_agent_details(script_name, agent_name, params={}, **kwargs):
    """
    Perform a GET call to get the details of a specific Network Analytics Engine agent on the device.

    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent that will have details returned
    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing the details of the specified NAE script on the device
    """
    target_url = kwargs["url"] + "system/nae_scripts/%s/nae_agents/%s" % (script_name, agent_name)
    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting details of NAE agent %s for script %s on device failed with status code %d: %s"
              % (agent_name, script_name, response.status_code, response.text))
        nae_script_details = {}
    else:
        logging.info("SUCCESS: Getting details of NAE agent %s for script %s on device succeeded"
                     % (agent_name, script_name))
        nae_script_details = response.json()

    return nae_script_details


def create_nae_agent(script_name, agent_name, agent_parameters={}, disabled=False, **kwargs):
    """
    Perform a POST call to create a Network Analytics Engine agent for the specified script. This function will also
    take in the agent name, as well as a dictionary of parameters that are specific to the agent.

    *Note that upon initial check in, creating an NAE agent in v10.04 is not working.
    *Note that encrypted parameters are not currently supported in this function.
    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent
    :param agent_parameters: Dictionary of optional parameters for the agent. The key:value pairing is based on the
        agent parameter name and values to be passed in.  Any parameters not mentioned will be set to the default value
        for the given parameter, specified in the NAE script.
    :param disabled: Boolean to determine if the agent is disabled upon creation.  By default, disabled is False,
        implying that the agent will be enabled upon creation.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _create_nae_agent_v1(script_name, agent_name, agent_parameters, disabled, **kwargs)
    else:  # Updated else for when version is v10.04
        return _create_nae_agent(script_name, agent_name, agent_parameters, disabled, **kwargs)


def _create_nae_agent_v1(script_name, agent_name, agent_parameters, disabled, **kwargs):
    """
    Perform a POST call to create a Network Analytics Engine agent for the specified script. This function will also
    take in the agent name, as well as a dictionary of parameters that are specific to the agent.

    *Note that encrypted parameters are not currently supported in this function.
    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent
    :param agent_parameters: Dictionary of optional parameters for the agent. The key:value pairing is based on the
        agent parameter name and values to be passed in.  Any parameters not mentioned will be set to the default value
        for the given parameter, specified in the NAE script.
    :param disabled: Boolean to determine if the agent is disabled upon creation.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    agent_list = get_all_nae_agents_of_script(script_name, **kwargs)

    if "/rest/v1/system/nae_scripts/%s/nae_agents/%s" % (script_name, agent_name) not in agent_list:
        agent_data = {
            "disabled": disabled,
            "encrypted_parameters_values": {},
            "name": agent_name,
            "parameters_values": agent_parameters
        }

        target_url = kwargs["url"] + "system/nae_scripts/%s/nae_agents" % script_name
        post_data = json.dumps(agent_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating NAE agent named '%s' from script '%s' failed with status code %d: %s"
                  % (agent_name, script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating NAE agent named '%s' from script '%s' succeeded"
                         % (agent_name, script_name))
            return True
    else:
        logging.info("SUCCESS: Agent creation not needed; NAE Agent named '%s' already exists on the system"
                     % agent_name)
        return True


def _create_nae_agent(script_name, agent_name, agent_parameters, disabled, **kwargs):
    """
    Perform a POST call to create a Network Analytics Engine agent for the specified script. This function will also
    take in the agent name, as well as a dictionary of parameters that are specific to the agent.

    *Note that encrypted parameters are not currently supported in this function.
    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent
    :param agent_parameters: Dictionary of optional parameters for the agent. The key:value pairing is based on the
        agent parameter name and values to be passed in.  Any parameters not mentioned will be set to the default value
        for the given parameter, specified in the NAE script.
    :param disabled: Boolean to determine if the agent is disabled upon creation.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    all_agents = get_all_nae_agents_of_script(script_name, **kwargs)

    if agent_name not in all_agents:
        agent_data = {
            "disabled": disabled,
            "encrypted_parameters_values": {},
            "name": agent_name,
            "parameters_values": agent_parameters
        }

        target_url = kwargs["url"] + "system/nae_scripts/%s/nae_agents" % script_name
        post_data = json.dumps(agent_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating NAE agent named '%s' from script '%s' failed with status code %d: %s"
                  % (agent_name, script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating NAE agent named '%s' from script '%s' succeeded"
                         % (agent_name, script_name))
            return True
    else:
        logging.info("SUCCESS: Agent creation not needed; NAE Agent named '%s' already exists on the system"
                     % agent_name)
        return True


def update_nae_agent(script_name, agent_name, agent_parameters, disabled=False, **kwargs):
    """
    Perform a PUT call to update a specified Network Analytics Engine agent.  This function will take in a dictionary of
    parameters that are specific to the agent, as well as a boolean to update whether the agent is disabled or enabled.

    *Note that encrypted parameters are not currently supported in this function.
    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent
    :param agent_parameters: Dictionary of optional parameters for the agent.  The key:value pairing is based on the
        agent parameter name and values to be passed in.  Any parameters not mentioned will be set to the default value
        for the given parameter, specified in the NAE script.
    :param disabled: Boolean to determine if the agent is disabled upon updating.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _update_nae_agent_v1(script_name, agent_name, agent_parameters, disabled, **kwargs)
    else:  # Updated else for when version is v10.04
        return _update_nae_agent(script_name, agent_name, agent_parameters, disabled, **kwargs)


def _update_nae_agent_v1(script_name, agent_name, agent_parameters, disabled, **kwargs):
    """
    Perform a PUT call to update a specified Network Analytics Engine agent.  This function will take in a dictionary of
    parameters that are specific to the agent, as well as a boolean to update whether the agent is disabled or enabled.

    *Note that encrypted parameters are not currently supported in this function.
    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent
    :param agent_parameters: Dictionary of optional parameters for the agent.  The key:value pairing is based on the
        agent parameter name and values to be passed in.  Any parameters not mentioned will be set to the default value
        for the given parameter, specified in the NAE script.
    :param disabled: Boolean to determine if the agent is disabled upon updating.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    all_agents = get_all_nae_agents_of_script(script_name, **kwargs)
    all_agents_check = []
    # Create a new list with full uri stripped from agent list
    for agent in all_agents:
        all_agents_check.append(agent[(agent.rfind('/')+1):])

    if agent_name not in all_agents_check:
        logging.warning("FAIL: Updating NAE agent named '%s' failed. No agent with that name was found." % agent_name)
        return False
    else:
        current_agent_details = get_nae_agent_details(script_name, agent_name, **kwargs)
        new_parameters = {}
        # Create a new dictionary with keys stripped of full uri
        for parameter in current_agent_details['parameters_values']:
            key = parameter[(parameter.rfind('/')+1):]
            value = current_agent_details['parameters_values'][parameter]
            new_parameters[key] = value

        # Update the stripped dictionary with the key/value pairs passed in for the function
        new_parameters.update(agent_parameters)

        updated_agent_data = {
            "disabled": disabled,
            "encrypted_parameters_values": {},
            "parameters_values": new_parameters
        }

        target_url = kwargs["url"] + "system/nae_scripts/%s/nae_agents/%s" % (script_name, agent_name)
        put_data = json.dumps(updated_agent_data, sort_keys=True, indent=4)

        response = kwargs["s"].put(target_url, data=put_data, verify=False)

        if not common_ops._response_ok(response, "PUT"):
            logging.warning("FAIL: Updating NAE agent named '%s' from script '%s' failed with status code %d: %s"
                  % (agent_name, script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Updating NAE agent named '%s' from script '%s' succeeded"
                         % (agent_name, script_name))
            return True


def _update_nae_agent(script_name, agent_name, agent_parameters, disabled, **kwargs):
    """
    Perform a PUT call to update a specified Network Analytics Engine agent.  This function will take in a dictionary of
    parameters that are specific to the agent, as well as a boolean to update whether the agent is disabled or enabled.

    *Note that encrypted parameters are not currently supported in this function.
    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent
    :param agent_parameters: Dictionary of optional parameters for the agent.  The key:value pairing is based on the
        agent parameter name and values to be passed in.  Any parameters not mentioned will be set to the default value
        for the given parameter, specified in the NAE script.
    :param disabled: Boolean to determine if the agent is disabled upon updating.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    all_agents = get_all_nae_agents_of_script(script_name, **kwargs)

    if agent_name not in all_agents:
        logging.warning("FAIL: Updating NAE agent named '%s' failed. No agent with that name was found." % agent_name)
        return False
    else:
        current_agent_details = get_nae_agent_details(script_name, agent_name, **kwargs)
        new_parameters = {}
        # Create a new dictionary with keys stripped of full uri
        for parameter in current_agent_details['parameters_values']:
            key = parameter[(parameter.rfind('/')+1):]
            value = current_agent_details['parameters_values'][parameter]
            new_parameters[key] = value

        # Update the stripped dictionary with the key/value pairs passed in for the function
        new_parameters.update(agent_parameters)

        updated_agent_data = {
            "disabled": disabled,
            "encrypted_parameters_values": {},
            "parameters_values": new_parameters
        }

        target_url = kwargs["url"] + "system/nae_scripts/%s/nae_agents/%s" % (script_name, agent_name)
        put_data = json.dumps(updated_agent_data, sort_keys=True, indent=4)

        response = kwargs["s"].put(target_url, data=put_data, verify=False)

        if not common_ops._response_ok(response, "PUT"):
            logging.warning("FAIL: Updating NAE agent named '%s' from script '%s' failed with status code %d: %s"
                  % (agent_name, script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Updating NAE agent named '%s' from script '%s' succeeded"
                         % (agent_name, script_name))
            return True


def delete_nae_agent(script_name, agent_name, **kwargs):
    """
    Perform a DELETE call to remove a Network Analytics Engine agent for the specified script.

    :param script_name: Alphanumeric String of the name of the script
    :param agent_name: Alphanumeric String of the name of the agent
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Boolean True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        agent_check = "/rest/v1/system/nae_scripts/%s/nae_agents/%s" % (script_name, agent_name)
    else:  # Updated else for when version is v10.04
        agent_check = agent_name

    all_agents = get_all_nae_agents_of_script(script_name, **kwargs)

    if agent_check not in all_agents:
        logging.info("SUCCESS: Agent deletion not needed; NAE Agent named '%s' does NOT exist on the system"
                     % agent_name)
        return True

    else:
        target_url = kwargs["url"] + "system/nae_scripts/%s/nae_agents/%s" % (script_name, agent_name)
        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting NAE agent named '%s' based on script '%s' failed with status code %d: %s"
                  % (agent_name, script_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting NAE agent named '%s' based on script '%s' succeeded"
                         % (agent_name, script_name))
            return True



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops

import json
import logging


def get_evpn_info(**kwargs):
    """
    Perform a GET call to receive the EVPN information on the system

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Information
    """
    if kwargs["url"].endswith("/v1/"):
        target_url = kwargs["url"] + "system/evpns"
    else:
        # Else logic designed for v10.04 and later
        target_url = kwargs["url"] + "system/evpn"

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.info("SUCCESS: Getting EVPN information replied with status code %d, no EVPN information exists"
              % response.status_code)
        evpn_info = []
    else:
        logging.info("SUCCESS: Getting EVPN information succeeded")
        evpn_info = response.json()

    return evpn_info


def create_evpn_instance(**kwargs):
    """
    Perform POST calls to create an EVPN instance

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    current_evpn = get_evpn_info(**kwargs)

    if current_evpn:
        logging.info("SUCCESS: No need to create EVPN instance since it already exists")
        return True
    else:
        evpn_data = {}

        if kwargs["url"].endswith("/v1/"):
            target_url = kwargs["url"] + "system/evpns"
        else:
            # Else logic designed for v10.04 and later
            target_url = kwargs["url"] + "system/evpn"

        post_data = json.dumps(evpn_data, sort_keys=True, indent=4)
        response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating EVPN Instance failed with status code %d: %s"
                            % (response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating EVPN Instance succeeded")
            return True


def delete_evpn_instance(**kwargs):
    """
    Perform DELETE calls to remove an EVPN instance

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    current_evpn = get_evpn_info(**kwargs)

    if current_evpn:
        if kwargs["url"].endswith("/v1/"):
            target_url = kwargs["url"] + "system/evpns"
        else:
            # Else logic designed for v10.04 and later
            target_url = kwargs["url"] + "system/evpn"
        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting EVPN Instance failed with status code %d: %s"
                            % (response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting EVPN Instance succeeded")
            return True
    else:
        logging.info("SUCCESS: No need to delete EVPN instance since it doesn't exist")
        return True


def get_evpn_vlan_list(**kwargs):
    """
    Perform a GET call to receive a list of VLANs associated with the EVPN instance

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of EVPN VLANs
    """
    target_url = kwargs["url"] + "system/evpns/evpn_vlans"

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list of all EVPN VLANs failed with status code %d: %s"
                        % (response.status_code, response.text))
        evpn_vlan_list = []
    else:
        logging.info("SUCCESS: Getting list of all EVPN VLANs succeeded")
        evpn_vlan_list = response.json()

    return evpn_vlan_list


def add_evpn_vlan(vlan_id, export_route=["auto"], import_route=["auto"], rd="auto", **kwargs):
    """
    Perform POST call to create an EVPN VLAN association
    Note that this functions has logic that works for both v1 and v10.04

    :param vlan_id: Integer representing the VLAN ID
    :param export_route: List of route targets to be exported from the VLAN in ASN:nn format, or auto.
    :param import_route: List of route targets to be imported from the VLAN in ASN:nn format, or auto.
    :param rd: Alphanumeric EVPN RD in ASN:nn format or IP:nn format, or auto.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    evpn_data = {
        "export_route_targets": export_route,
        "import_route_targets": import_route,
        "rd": rd
    }
    if kwargs["url"].endswith("/v1/"):
        evpn_data.update({'vlan': '/rest/v1/system/vlans/%s' % vlan_id})
        target_url = kwargs["url"] + "system/evpns/evpn_vlans"
    else:
        # Else logic designed for v10.04 and later
        evpn_data.update({'vlan': '/rest/v10.04/system/vlans/%s' % vlan_id})
        target_url = kwargs["url"] + "system/evpn/evpn_vlans"

    post_data = json.dumps(evpn_data, sort_keys=True, indent=4)
    response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

    if not common_ops._response_ok(response, "POST"):
        logging.warning("FAIL: Creating EVPN VLAN '%s' association failed with status code %d: %s"
                        % (vlan_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Creating EVPN VLAN '%s' association succeeded" % vlan_id)
        return True



#!/usr/bin/env python3

# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

"""
This workflow performs the following steps:
1. Delete DHCP helpers from SVI
      Ex:
        interface vlan999
            no ip helper-address 1.1.1.1
            no ip helper-address 2.2.2.2

2. Delete SVI
      Ex:
        no interface vlan 999

3. Delete VLAN
      Ex:
        no vlan 999

4. Initialize L2 interface
      Ex:
        interface 1/1/20
            no shutdown
            no routing
            vlan access 1

Preconditions:
Must have run the configure_l2_l3_vlans workflow or have the equivalent settings.
"""

from requests.packages.urllib3.exceptions import InsecureRequestWarning
import requests
import os
import sys
import logging
import getpass

logging.basicConfig(level=logging.INFO)
dirpath = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
sys.path.append(dirpath)
sys.path.append(os.path.join(dirpath, "pyaoscx"))
sys.path.append(os.path.join(dirpath, "cx_utils"))

from pyaoscx import session
from pyaoscx import dhcp
from pyaoscx import interface
from pyaoscx import vlan

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


def main():
    switchip = input("Switch IP Address: ")

    username = input("Switch login username: ")

    password = getpass.getpass("Switch login password: ")

    bypassproxy = False  # Set to 'True' to bypass proxy and communicate directly with device.

    if bypassproxy:
        os.environ['no_proxy'] = switchip
        os.environ['NO_PROXY'] = switchip

    version = 'v1'  # Set to 'v10.04' if running code v10.04 or later, otherwise set to 'v1'

    base_url = "https://{0}/rest/{1}/".format(switchip, version)
    try:
        session_dict = dict(s=session.login(base_url, username, password), url=base_url)

        # Delete all DHCP relays for interface
        dhcp.delete_dhcp_relays('vlan999', "default", **session_dict)

        # Delete VLAN and SVI
        vlan.delete_vlan_and_svi(999, 'vlan999', **session_dict)

        # Initialize L2 interface
        interface.initialize_interface('1/1/20', **session_dict)

    except Exception as error:
        print('Ran into exception: {}. Logging out..'.format(error))
    session.logout(**session_dict)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

"""
This workflow performs the following steps:
1. Create VLAN
      Ex:
        vlan 999
            description For LAB 999

2. Create SVI
      Ex:
        interface vlan999
            description ### SVI for LAB999 ###
            ip address 10.10.10.99/24

3. Add DHCP helpers for SVI
      Ex:
        interface vlan999
            description ### SVI for LAB999 ###
            ip address 10.10.10.99/24
            ip helper-address 1.1.1.1
            ip helper-address 2.2.2.2

3. Create L2 interface
    a. Create the interface
    b. Enable the interface
    c. Set VLAN mode to 'access'
    d. Set VLAN as untagged VLAN
      Ex:
        interface 1/1/20
            no shutdown
            no routing
            vlan access 999



Preconditions:
None
"""

from requests.packages.urllib3.exceptions import InsecureRequestWarning
import requests
import os
import sys
import logging
import getpass

logging.basicConfig(level=logging.INFO)
dirpath = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
sys.path.append(dirpath)
sys.path.append(os.path.join(dirpath, "pyaoscx"))
sys.path.append(os.path.join(dirpath, "cx_utils"))

from pyaoscx import session
from pyaoscx import vlan
from pyaoscx import interface
from pyaoscx import dhcp

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


def main():
    switchip = input("Switch IP Address: ")

    username = input("Switch login username: ")

    password = getpass.getpass("Switch login password: ")

    bypassproxy = False  # Set to 'True' to bypass proxy and communicate directly with device.

    if bypassproxy:
        os.environ['no_proxy'] = switchip
        os.environ['NO_PROXY'] = switchip

    version = 'v1'  # Set to 'v10.04' if running code v10.04 or later, otherwise set to 'v1'

    base_url = "https://{0}/rest/{1}/".format(switchip, version)
    try:
        session_dict = dict(s=session.login(base_url, username, password), url=base_url)

        vlan.create_vlan_and_svi(999, 'VLAN999', 'vlan999', 'vlan999',
                                 'For LAB 999', '10.10.10.99/24', vlan_port_desc='### SVI for LAB999 ###',
                                 **session_dict)

        # Add DHCP helper IPv4 addresses for SVI
        dhcp.add_dhcp_relays('vlan999', "default", ['1.1.1.1', '2.2.2.2'], **session_dict)

        # Add a new entry to the Port table if it doesn't yet exist
        interface.add_l2_interface('1/1/20', **session_dict)

        # Update the Interface table entry with "user-config": {"admin": "up"}
        interface.enable_disable_interface('1/1/20', **session_dict)

        # Set the L2 port VLAN mode as 'access'
        vlan.port_set_vlan_mode('1/1/20', "access", **session_dict)

        # Set the access VLAN on the port
        vlan.port_set_untagged_vlan('1/1/20', 999, **session_dict)

    except Exception as error:
        print('Ran into exception: {}. Logging out..'.format(error))
    session.logout(**session_dict)


if __name__ == '__main__':
    main()
